angular.module('starter.controllers', [])

.controller('AppController', function($scope, $ionicModal, $timeout, $rootScope, $location, consts, UserService) {
  
  $rootScope.baseUrl = consts.baseUrl;
    
  $rootScope.baseAppUrl = consts.baseAppUrl;

  $rootScope.user = {};
    
  $rootScope.isAuthenticated = UserService.getIsAuthenticated();

  $scope.authMode = 'login';

  $scope.formSubmitted = false;
  
  $scope.formError = '';

  // Form data for the login modal
  $scope.loginData = {};

  // Create the login modal that we will use later
  $ionicModal.fromTemplateUrl('views/auth.html', {
    scope: $scope
  }).then(function(modal) {
    $scope.modal = modal;
  });

  // Triggered in the login modal to close it
  $scope.closeAuth = function() {
    $scope.modal.hide();
  };

  // Open the login modal
  $scope.openAuth = function(mode) {
    $scope.authMode = mode;
    $scope.modal.show();
  };

  $scope.changeAuthMode = function(mode) {
    $scope.authMode = mode;
    
    if(!$scope.$$phase) {
      $scope.$apply();
    }
  };

  $scope.doLogin = function() {
    $scope.formSubmitted = true;
    $scope.formError = '';
    UserService.login($scope.loginData.email, $scope.loginData.password).then(function(res) {
      if(res.data.success) {
        UserService.getUserInfoRequest();
        $scope.closeAuth();
      }
      else {
        $scope.formError = res.data.message;
      }
      $scope.formSubmitted = false;
    });
  };

  $scope.doSignup = function() {
    $scope.formSubmitted = true;
    $scope.formError = '';
    UserService.signup($scope.loginData.regName, $scope.loginData.regEmail, $scope.loginData.regPassword).then(function(res) {
      if(res.data.success) {
        UserService.getUserInfoRequest();
        $scope.closeAuth();
      }
      else {
        $scope.formError = res.data.message;
      }
      $scope.formSubmitted = false;
    });
  };

  $scope.doLogout = function() {
    UserService.logout();
  };

  $scope.getItemUrl = function(item) {
    return item.url + '#markticle=' + item.elmType + ':' + item.text;
  };

  $scope.getTimeAgo = function(time) {
    return moment(time).fromNow();
  };

  $scope.getEncodedUrl = function(url) {
    return encodeURIComponent(url);
  };

  // Try to get user info
  UserService.getUserInfoRequest();

  $rootScope.$on('authChanged', function(event) {
    $rootScope.user = UserService.getUserInfo();
    $rootScope.isAuthenticated = UserService.getIsAuthenticated();

    if(!$rootScope.$$phase) {
      $rootScope.$apply();
    }
  });
})

.controller('MarksController', function($scope, $rootScope, $stateParams, MarksService, TagsService) {
  
  $rootScope.searchQuery = '';

  $scope.markItems = [];

  $scope.singleTagId = $stateParams.tagId ? $stateParams.tagId : null;
  
  $scope.singleTagName = $stateParams.tagName ? $stateParams.tagName : null;

  $rootScope.isLoading = true;

  $scope.removeItem = function(idx, url) {
    $scope.markItems.splice(idx, 1);
    MarksService.remove(url);

    if(!$scope.$$phase) {
      $scope.$apply();
    }
  };

  $scope.getMarks = function() {
    if(!$rootScope.isAuthenticated) {
      $scope.markItems = [];
      $rootScope.isLoading = false;
      return;
    }

    $rootScope.isLoading = true;

    var callback = function(res) {
      if(res && res.data.success) {
        var data = JSON.parse(res.data.data) || [];

        $scope.markItems = data;
      }
      
      // Set is loading to false
      $rootScope.isLoading = false;

      if(!$scope.$$phase) {
        $scope.$apply();
      }
    };

    // Fetch marks
    if(!$scope.singleTagId) {
      MarksService.get().then(callback);
    }
    else {
      TagsService.getSingleTagItems($scope.singleTagId).then(callback);
    }
  };

  // Get current items from localstorage
  $scope.getMarks();

  $rootScope.$on('authChanged', function(event) {
    $scope.getMarks();

    if(!$rootScope.$$phase) {
      $rootScope.$apply();
    }
  });
})

.controller('TagsController', function($scope, $rootScope, TagsService) {
  
  $rootScope.searchQuery = '';

  $scope.tagItems = [];

  $rootScope.isLoading = true;

  $scope.getTags = function() {
    if(!$rootScope.isAuthenticated) {
      $scope.tagItems = [];
      $rootScope.isLoading = false;
      return;
    }

    $rootScope.isLoading = true;

    // Fetch marks
    TagsService.get().then(function(res) {
      if(res && res.data.success) {
        var data = JSON.parse(res.data.data) || [];

        $scope.tagItems = data;
      }
      
      // Set is loading to false
      $rootScope.isLoading = false;

      if(!$scope.$$phase) {
        $scope.$apply();
      }
    });
  };

  // Get current items from localstorage
  $scope.getTags();

  $rootScope.$on('authChanged', function(event) {
    $scope.getTags();

    if(!$rootScope.$$phase) {
      $rootScope.$apply();
    }
  });
})

.controller('HistoryController', function($scope, $rootScope, HistoryService) {
  
  $rootScope.searchQuery = '';

  $scope.markItems = [];

  $rootScope.isLoading = true;

  $scope.removeItem = function(idx, url) {
    $scope.markItems.splice(idx, 1);
    HistoryService.remove(url);

    if(!$scope.$$phase) {
      $scope.$apply();
    }
  };

  $scope.getMarks = function() {
    if(!$rootScope.isAuthenticated) {
      $scope.markItems = [];
      $rootScope.isLoading = false;
      return;
    }

    $rootScope.isLoading = true;

    // Fetch marks
    HistoryService.get().then(function(res) {
      if(res && res.data.success) {
        var data = JSON.parse(res.data.data) || [];

        $scope.markItems = data;
      }
      
      // Set is loading to false
      $rootScope.isLoading = false;

      if(!$scope.$$phase) {
        $scope.$apply();
      }
    });
  };

  // Get current items from localstorage
  $scope.getMarks();

  $rootScope.$on('authChanged', function(event) {
    $scope.getMarks();

    if(!$rootScope.$$phase) {
      $rootScope.$apply();
    }
  });
})

.controller('SettingsController', function($scope, $rootScope, SettingsService) {
  
  $scope.settings = SettingsService.get();

  $scope.backgroundColors = [
    {
      name: 'Yellow',
      value: 'ffd84b'
    },
    {
      name: 'Blue',
      value: '0d335a'
    },
    {
      name: 'Light blue',
      value: '6cb5f4'
    },
    {
      name: 'Red',
      value: 'a51f2c'
    },
    {
      name: 'Purple',
      value: '852599'
    },
    {
      name: 'Pink',
      value: 'f570ce'
    },
    {
      name: 'Green',
      value: '50ca20'
    },
    {
      name: 'Black',
      value: '000000'
    },
    {
      name: 'Orange',
      value: 'f9660d'
    }
  ];

  $scope.textColors = [
    {
      name: 'dark',
      value: '333333'
    },
    {
      name: 'light',
      value: 'ffffff'
    }
  ];

  $scope.updateBackgroundColor = function() {
    SettingsService.set('markerBackground', $scope.settings.markerBackground);
  };

  $scope.updateMarkerColor = function() {
    SettingsService.set('markerColor', $scope.settings.markerColor);
  };

  $scope.updateScroll = function() {
    SettingsService.set('autoScroll', ($scope.settings.autoScroll === 'true' ? 'false' : 'true'));
  };

  // $scope.$watch('blockedDomains', function() {
  //   SettingsService.set('blockedDomains', $scope.settings.blockedDomains);
  // });
});
