angular.module('validation.engine', [])
  .value('validationTypes', '')
  .directive('validate', function (validationTypes) {
    return {
      require: '?ngModel',
      link: function (scope, element, attrs, ngModel) {
        var validationTypes = [];

        var emailRegex = /^[\w\-\+_]+(\.[\w\-\+_]+)*\@[\w\-\+_]+\.[\w\-\+_]+(\.[\w\-\+_]+)*$/;

        var errors = {
          empty: 'Field cannot remain empty',
          minLength: 'Minimum length is ',
          maxLength: 'Maximum length is ',
          select: '',
          invalidUrl: 'Invalid URL',
          invalidEmail: 'Invalid email address',
          specialChars: 'Special characters are not allowed',
          emails: 'One of the email addresses you provided is invalid'
        };

        var validateField = function (elm) {
          var error = null;
          for (var i = 0; i < validationTypes.length; i++) {
            if (error) { break; }
            // Triming value
            var value = elm.val().replace(/^\s+|\s+$/g, '');
            var validationType = validationTypes[i];
            if(validationType === 'empty') {
              // Empty fields
              if (value.length < 1) {
                error = errors.empty;
              }
            }
            else if(validationType === 'email') {
              if (value.search(emailRegex) < 0) {
                error = errors.invalidEmail;
              }
            }
            else if(validationType.indexOf('minLength') === 0) {
              var min = parseInt(validationTypes[i].split(':')[1] || 1);
              if (value.length < min) {
                error = errors.minLength + min;
              }
            }
            else if(validationType.indexOf('maxLength') === 0) {
              var max = parseInt(validationTypes[i].split(':')[1] || 1);
              if (value.length > max) {
                error = errors.maxLength + max;
              }
            }
            else if(validationType === 'specialChars') {
              if(!/^[A-z -]+$/.test(value)) {
                error = errors.specialChars;
              }
            }
            else if(validationType === 'emails') {
              var emails = value.split(/[ ,;\n]/);
              for(var x = 0; x < emails.length; x++) {
                var email = angular.element.trim(emails[x]);
                if (email !== '' && email.search(emailRegex) < 0) {
                  error = errors.emails;
                  break;
                }
              }
            }
          }
          return error;
        };

        scope.$watch(attrs.validate, function (types) {
          validationTypes = types.split('|');
        });

        element.on('blur', function () {
          var elm = angular.element(this);
          var field_container = elm.parent();

          if(elm.attr('disabled') === 'disabled') {
            return;
          }

          // Remove previus validation
          field_container.removeClass('field_error').removeClass('field_success').find('.error').remove();

          // Look for errors
          var error = validateField(elm);

          if (error) {
            field_container.addClass('field_error');
            angular.element('<span class="error">' + error + '</span>').appendTo(field_container);
          }
          else {
            field_container.addClass('field_success');
          }
        });

        element.on('keydown', function (e) {
          if (e.keyCode === 13) {
            angular.element(this).trigger('blur').focus();
          }
        });

        element.parents('form').on('submit', function(e) {
          element.trigger('blur').focus();
        });
      }
    };
  }
);