angular.module('starter.services', [])

.service('SettingsService', function($rootScope, $q) {
	var defaults = {
		autoScroll: 'true',
	    markerBackground: 'ffd84b',
	    markerColor: '333333'
	};

	var options = localStorage.getItem('markticle_settings') ? JSON.parse(localStorage.getItem('markticle_settings')) : {};
	options = angular.extend({}, defaults, options);

	return {
		
		get: function() {
			console.log(options);
			return options;
		},

		set: function(property, value) {
			options[property] = value;
			this.save();
		},

		save: function() {
			localStorage.setItem('markticle_settings', JSON.stringify(options));
		}

	};
})

.service('MarksService', function($rootScope, $q, $http, consts) {
	var baseUrl = consts.baseUrl + 'api/marks/';
	var requestCanceller = $q.defer();

	return {
		get: function() {
			var request = $http.get(baseUrl, {withCredentials: true});
      		return request;
		},

		remove: function(url) {
			var me = this;
      		var data = {
				url: url
			};
      
      		var request = $http.post(baseUrl + 'remove', data, {withCredentials: true});
      		return request;
		}
	};
})

.service('TagsService', function($rootScope, $q, $http, consts) {
	var baseUrl = consts.baseUrl + 'api/tags/';
	var requestCanceller = $q.defer();

	return {
		get: function() {
			var request = $http.get(baseUrl, {withCredentials: true});
      		return request;
		},

		getSingleTagItems: function(tagID) {
			var request = $http.get(baseUrl + 'single?tagID=' + tagID, {withCredentials: true});
      		return request;
		}
	};
})

.service('HistoryService', function($rootScope, $q, $http, consts) {
	var baseUrl = consts.baseUrl + 'api/history/';
	var requestCanceller = $q.defer();

	return {
		get: function() {
			var request = $http.get(baseUrl, {withCredentials: true});
			return request;
		},

		remove: function(url) {
			var me = this;
			var data = {
				url: url
			};

      		var request = $http.post(baseUrl + 'remove', data, {withCredentials: true});
			return request;
		}
	};
})

.service('UserService', function($rootScope, $q, $http, consts) {
	var baseUrl = consts.baseUrl + 'user/';
	var requestCanceller = $q.defer();
	var isAuthenticated = false;
  	var userInfo = {};

	return {
		
		getIsAuthenticated: function() {
			return isAuthenticated;
		},

		getUserInfo: function () {
			return userInfo;
		},

		getUserInfoRequest: function() {
			var request = $http.get(baseUrl + 'getUserInfo', {withCredentials: true});

	        request.then(function(result) {
	          if(result.data.success) {
	            isAuthenticated = true;
	            userInfo = angular.fromJson(result.data.data);
	          }
	          else {
	            isAuthenticated = false;
	            userInfo = {};
	          }
          	  $rootScope.$broadcast('authChanged');
	        });

	        return request;
		},

		login: function (email, password) {
	        var me = this;

	        var params = {
	          email: email,
	          password: password
	        };

	        var request = $http.post(baseUrl + 'login', params, {withCredentials: true});

	        request.then(function(result) {
	          if(result.data.success) {
	            me.getUserInfoRequest();
	            // AnalyticsService.track_event('Engagement Action', 'Signed In');
	          }
	        });

	        return request;
      	},

      signup: function(email, name, password) {
        var me = this;

        var params = {
          email: email,
          name: name,
          password: password
        };

        var request = $http.post(baseUrl + 'signup', params, {withCredentials: true});

        request.then(function(result) {
          if(result.data.success) {
            me.getUserInfoRequest();
            // AnalyticsService.track_event('Engagement Action', 'Signed Up');
          }
        });

        return request;
      },

      logout: function() {
      	var me = this;
      	
      	var request = $http.get(baseUrl + 'logout', {withCredentials: true});

      	request.then(function(result) {
          if(result.data.success) {
            me.getUserInfoRequest();
            // AnalyticsService.track_event('Engagement Action', 'Logged Out');
          }
        });

        return request;
      }

	};
});